#!/usr/bin/env python

import sys, getopt, cgi, traceback
from nws.client import NetWorkSpace, NwsServerException, NwsConnectException, NwsNoWorkSpaceException

styleBlock = '''\
<style>
  .error {
    color: red;
    margin: 0;
  }
  .info {
    color: blue;
    margin: 1em 0em 1em 0em;
  }
  body {
    border: 0;
    padding: 0;
    margin: 0;
    background-color: #efefef;
  }
  h4 {
    padding: 0.1em;
    background-color: #777;
    color: white;
    border-bottom: thin white dashed;
  }
  h5 {
    padding: 0.1em;
    background-color: #999;
    color: white;
    border-bottom: thin green dashed;
    border-top: thin green dashed;
  }
</style>
'''

def htmlReply(monName, wsName, varName=None, value=None, msg=None, vars=[]):
    reply = '''\
<html>
<head>
<title>Nws Utility</title>
%s
</head>
<body>
<h4>Nws Utility</h4>
''' % (styleBlock,)

    reply += '<div class="info"> ' + cgi.escape(msg) + ' </div>'

    reply += '''\
<form action="doit" method="get">
  <label for="varName">Variable Name:</label>
  <input type="text" name="varName" id="varName" value="">
  <label for="value">Value:</label>
  <input type="text" name="value" id="value" value="">
  <input type="hidden" name="op" id="op" value="showMonitor">
  <input type="hidden" name="monName" id="monName" value="%s">
  <input type="hidden" name="wsName" id="wsName" value="%s">
  <input type="submit" value="store">
</form>
''' % (cgi.escape(monName), cgi.escape(wsName),)

    if vars:
        reply += 'Variables: <ul>'
        for v in vars:
            reply += '<li>' + cgi.escape(v) + '</li>'
        reply += '</ul>'

    reply += '''\
<p>
<h5><a href="doit?op=listVars&wsName=%s"> List variables in %s </a></h5>
<p>
<h5><a href="doit?op=listWss"> List netWorkSpaces </a></h5>
</body>
</html>
''' % (cgi.escape(wsName), cgi.escape(wsName))

    return reply

def htmlError(msg, e):
    return '''\
<html>
<head>
<title>Error</title>
%s
</head>
<body>
<h4>Error</h4>
<div class="error"> %s: %s </div>
<p>
<h5><a href="doit?op=listWss"> List netWorkSpaces </a></h5>
</body>
</html>
''' % (styleBlock, cgi.escape(msg), cgi.escape(str(e)))

class Args:
    def __init__(self, **kw):
        for k, v in kw.items():
            setattr(self, k, v)

def utility(host, port):
    monName = 'Nws Utility'
    mon = NetWorkSpace(monName, host, port)

    while 1:
        args = Args(wsName=None, varName=None, value=None, replyVarName='reply')
        msg = ''

        try:
            n = int(mon.fetch('request'))

            for i in range(n):
                a = mon.fetch('request')
                k, v = a.split('=', 1)
                if hasattr(args, k):
                    setattr(args, k, v)
                else:
                    print >> sys.stderr, 'error: received unknown argument ' + k

            ws = mon.server.useWs(args.wsName, create=False)

            if args.varName:
                # if not created, make it's mode 'single'
                try: ws.declare(args.varName, 'single')
                except: pass

                ws.store(args.varName, str(args.value))
                msg = 'Set variable "%s" to "%s"' % (args.varName, str(args.value))

            varinfo = ws.listVars().split('\n')
            vars = [v.split('\t')[0] for v in varinfo]

            t = htmlReply(monName, args.wsName, args.varName, args.value, msg, vars)
        except ImportError, e:
            t = htmlError('Import error', e)
        except NwsNoWorkSpaceException, e:
            t = htmlError('Error opening workspace', e)

        mon.store(args.replyVarName, "0")  # no reply headers to set
        mon.store(args.replyVarName, t)

def main(argv):
    host = 'localhost'
    port = 8765

    try:
        opts, args = getopt.getopt(argv, 'h:p:')

        for opt, arg in opts:
            if opt == '-h':
                host = arg
            elif opt == '-p':
                port = int(arg)
            else:
                raise 'internal error: out-of-sync with getopt'

        if len(args) > 0:
            print >> sys.stderr, 'ignoring unused arguments:', args

        utility(host, port)
    except getopt.GetoptError, e:
        print >> sys.stderr, e.msg
        sys.exit(1)
    except ValueError, e:
        print >> sys.stderr, 'option %s requires an integer argument' % opt
        sys.exit(1)
    except NwsConnectException, e:
        # the server isn't running
        print >> sys.stderr, e.args[0]
    except (KeyboardInterrupt, NwsServerException, SystemExit):
        # either the user or the server is telling us to shutdown
        pass
    except:
        traceback.print_exc()

if __name__ == '__main__':
    main(sys.argv[1:])
