#!/bin/sh
#
# Copyright (c) 2005-2006, Scientific Computing Associates, Inc.
#
# NetWorkSpaces is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA
#

PythonProg=${PythonProg:-'python'}
cd ${PythonSleighWorkingDir:-'/tmp'}
LogDir=${PythonSleighLogDir:-'/tmp'}
if [ -n "${PythonSleighWorkerOut}" ]; then
    PythonSleighVerbose='1'
    WorkerOut="${LogDir}/`basename ${PythonSleighWorkerOut}`"
else
    PythonSleighVerbose='0'
    WorkerOut='/dev/null'
fi
export PythonSleighVerbose

# compute engine
$PythonProg <<'EOF' > ${WorkerOut} 2>&1 &

import os, sys, signal
modulePath = os.environ.get('PythonSleighModulePath')
if modulePath: sys.path[1:1] = modulePath.split(os.pathsep)
from nws.sleigh import cmdLaunch

def handler(sig, frame):
    sys.exit(128 + sig)

signal.signal(signal.SIGTERM, handler)

cmdLaunch(int(os.environ.get('PythonSleighVerbose', '0')))

EOF

PythonCEPid=$!
export PythonCEPid

# sentinel
$PythonProg <<'EOF' > ${LogDir}/PythonSleighSentinelLog_${UID}_${PythonSleighID} 2>&1 &

import os, sys, signal
modulePath = os.environ.get('PythonSleighModulePath')
if modulePath: sys.path[1:1] = modulePath.split(os.pathsep)
from nws.client import NetWorkSpace

def handler(sig, frame):
    sys.exit(128 + sig)

signal.signal(signal.SIGTERM, handler)
Env = os.environ

nws = NetWorkSpace(Env['PythonSleighNwsName'], Env['PythonSleighNwsHost'],
        int(Env['PythonSleighNwsPort']), useUse=True, create=False)

print "waiting for sleigh to be stopped"

try: 
    nws.find('Sleigh ride over')
    nws.store('bye', 1)
except:
    try: nws.store('bye', 101)
    except: pass

print "sentinel returning"

EOF

SentinelPid=$!
export SentinelPid

# wait for the sentinel process to exit,
# and then kill the worker process
wait $SentinelPid
kill $PythonCEPid 2> /dev/null
